==> ktexteditor-5.63.0.tar.xz: ktexteditor-5.63.0/src/completion/expandingtree/expandingtree.cpp <==
/*  This file is part of the KDE libraries and the Kate part.
 *
 *  Copyright (C) 2007 David Nolden <david.nolden.kdevelop@art-master.de>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#include "expandingtree.h"

#include <QTextLayout>
#include <QTextDocument>
#include <QHeaderView>
#include <QAbstractTextDocumentLayout>
#include <QPainter>
#include "katepartdebug.h"
#include "expandingwidgetmodel.h"

ExpandingTree::ExpandingTree(QWidget *parent)
    : QTreeView(parent)
{
    m_drawText.documentLayout()->setPaintDevice(this);
    setUniformRowHeights(false);
    header()->setMinimumSectionSize(0);
}

void ExpandingTree::drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QTreeView::drawRow(painter, option, index);

    const ExpandingWidgetModel *eModel = qobject_cast<const ExpandingWidgetModel *>(model());
    if (eModel && eModel->isPartiallyExpanded(index)) {
        QRect rect = eModel->partialExpandRect(index);
        if (rect.isValid()) {
            painter->fillRect(rect, QBrush(0xffffffff));

            QAbstractTextDocumentLayout::PaintContext ctx;
            // since arbitrary HTML can be shown use a black on white color scheme here
            ctx.palette = QPalette(Qt::black, Qt::white);
            ctx.clip = QRectF(0, 0, rect.width(), rect.height());
            ;
            painter->setViewTransformEnabled(true);
            painter->translate(rect.left(), rect.top());

            m_drawText.setHtml(eModel->partialExpandText(index));
            m_drawText.setPageSize(QSizeF(rect.width(), rect.height()));
            m_drawText.documentLayout()->draw(painter, ctx);

            painter->translate(-rect.left(), -rect.top());
        }
    }
}

int ExpandingTree::sizeHintForColumn(int column) const
{
    return columnWidth(column);
}

==> ktexteditor_5.68.0.orig.tar.xz: ktexteditor-5.68.0/src/completion/expandingtree/expandingtree.cpp <==
/*  SPDX-License-Identifier: LGPL-2.0-or-later

    Copyright (C) 2007 David Nolden <david.nolden.kdevelop@art-master.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "expandingtree.h"

#include "expandingwidgetmodel.h"
#include "katepartdebug.h"
#include <QAbstractTextDocumentLayout>
#include <QHeaderView>
#include <QPainter>
#include <QTextDocument>
#include <QTextLayout>

ExpandingTree::ExpandingTree(QWidget *parent)
    : QTreeView(parent)
{
    m_drawText.documentLayout()->setPaintDevice(this);
    setUniformRowHeights(false);
    header()->setMinimumSectionSize(0);
}

void ExpandingTree::drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QTreeView::drawRow(painter, option, index);

    const ExpandingWidgetModel *eModel = qobject_cast<const ExpandingWidgetModel *>(model());
    if (eModel && eModel->isPartiallyExpanded(index)) {
        QRect rect = eModel->partialExpandRect(index);
        if (rect.isValid()) {
            painter->fillRect(rect, QBrush(0xffffffff));

            QAbstractTextDocumentLayout::PaintContext ctx;
            // since arbitrary HTML can be shown use a black on white color scheme here
            ctx.palette = QPalette(Qt::black, Qt::white);
            ctx.clip = QRectF(0, 0, rect.width(), rect.height());
            ;
            painter->setViewTransformEnabled(true);
            painter->translate(rect.left(), rect.top());

            m_drawText.setHtml(eModel->partialExpandText(index));
            m_drawText.setPageSize(QSizeF(rect.width(), rect.height()));
            m_drawText.documentLayout()->draw(painter, ctx);

            painter->translate(-rect.left(), -rect.top());
        }
    }
}

int ExpandingTree::sizeHintForColumn(int column) const
{
    return columnWidth(column);
}
